clear;

global W N PROFILES NEXT_P; 

OUTPUT = 1;
NEXT_P = [2 1];

W = 2; % number of actions
N = 2; % number of players
PROFILES = encodei(ones(N,1)*W); % number of possible profiles


MAX_ITER = 20; % maximal number of iterations of the main loop (can probably be 5)

D(:,:) = [.01 .24; .04 .02; .05 0; .11 0]; % payoff matrix

T = 1; % deadline of the game (presumes that the biggest t_star is < T)

% ---------------------------------------------------------------------------------
% beginning of the program
% ---------------------------------------------------------------------------------

V = zeros(PROFILES,N,MAX_ITER); % continuation values
F = zeros(PROFILES,N,MAX_ITER); % flags of immediate active moves
t = zeros(MAX_ITER,1);     % 

% initializiation
n    = 1; % iteration counter
t(n) = T; 
V(:,:,n) = D(:,:); % final values
EQUAL_T  = 0; % flag for equal critical times for both players
MULTIPLE = 0; % flag for multiplicity of equilibria because of indifference

% ---------------------------------------------------------------------------------
% main loop of the algorithm
% ---------------------------------------------------------------------------------

while 1==1, 
   
   n = n+1; % stages counter
   [p_star a_star t(n)] = findmax(V(:,:,n-1),F(:,:,n-1),t(n-1));
   
   % check for termination
   
   EQUAL_T = min(p_star) < max(p_star); 
   MULTIPLE = (t(n)==0);   
   if EQUAL_T | MULTIPLE,
      break;
   end;   
   p_star = p_star(1); % at this point there must be a unique p_star
   
   num_max = length(a_star);
   
   % update F
   
   for i = 1:PROFILES,
      for p = 1:N, 
         
         F(i,p,n) = F(i,p,n-1);

         indic = 0;
         for j = 1:num_max,
            indic = indic | ((i == a_star(j)) & p == (p_star));
         end;
         if indic,
            F(i,p,n) = 1;
         end;   
         
         indic = 0;
         for j = 1:num_max,
            indic = indic | ((i == a_star(j)) & p ~= (p_star));
         end;
         if indic & (n==3 | F(my_neg(my_neg(i,1),2),p,n-1)==1),
            F(i,p,n) = 0;
         end;   
         
         indic = 0;
         for j = 1:num_max,
            indic = indic | ((i == my_neg(a_star(j),p)) & p ~= (p_star));
         end;
         if indic,
            F(i,p,n) = 0;
         end;   
         
      end;
   end;      
   
   % update V
   
   V(:,:,n) = newV(F(:,:,n-1),V(:,:,n-1),NEXT_P(p_star),t(n));
   
   % check for termination
   
   if sum(sum(F(:,:,n))) == PROFILES*N*(W-1)/W,
      n_bar = n;
      t(n+1) = 0;
      % V(:,:,n+1) = newV(V(:,:,n),F(:,:,n),1,0);
      break;
   end;   
   
end;   

% ---------------------------------------------------------------------------------
% output
% ---------------------------------------------------------------------------------
   
   if EQUAL_T == 0 & MULTIPLE == 0,
   
      initial = find(sum(F(:,:,n_bar)')==0);
      final   = initial;
      for i = n_bar:-1:1,
         if F(initial,1,i) == 1,
            switch_p = 1;
            switch_n = i+1;
            final = my_neg(initial,1);
            break;
         elseif F(initial,2,i) == 1,
            switch_p = 2;
            switch_n = i+1;
            final = my_neg(initial,2);
            break;
         end;
      end;   
      
      if OUTPUT,
      
      disp(' ');
      for i = n_bar:-1:1,
         disp(sprintf('Block number %d from %3.3f to %3.3f',i,t(i+1),t(i)));
         disp(sprintf('-----------------------------'));
         disp(sprintf('Active Moves by Player 1 - %d %d %d %d',F(:,1,i)));
         disp(sprintf('Active Moves by Player 2 - %d %d %d %d',F(:,2,i)));
         disp(' ');
         disp(sprintf('Cont Values for Player 1 - %3.3f %3.3f %3.3f %3.3f',V(:,1,i)));
         disp(sprintf('Cont Values for Player 2 - %3.3f %3.3f %3.3f %3.3f',V(:,2,i)));
         disp(' ');
      end;   
         
      disp(sprintf('\nthe game is:\n\n   %3.3f,%3.3f | %3.3f,%3.3f\n   ------------+------------\n   %3.3f,%3.3f | %3.3f,%3.3f\n', ...
         D(encodei([1 1]'),1),D(encodei([1 1]'),2),D(encodei([1 2]'),1),D(encodei([1 2]'),2), ...
         D(encodei([2 1]'),1),D(encodei([2 1]'),2),D(encodei([2 2]'),1),D(encodei([2 2]'),2)));
      disp(sprintf('final values are [%3.3f,%3.3f]',V(initial,:,n_bar)));
      disp(sprintf('initial position are [%d,%d]',decodei(initial)));
      if final ~= initial,
         disp(sprintf('final position switched to [%d,%d] at t = %3.3f', ...
            decodei(final),t(switch_n)));
      end;   
      
      end; % OUTPUT
      
   elseif MULTIPLE == 1,
      disp('One of the player is indifferent --> Multiplicity !!!');
   else,
      disp('Equal critical times for both players !!! No Order-Inavriance !!!');
   end; % if EQUAL_T
  
      
